package net.minecraft.src.ic2.api;

import net.minecraft.src.Entity;
import net.minecraft.src.EntityLiving;
import net.minecraft.src.EntityPlayer;
import net.minecraft.src.ItemStack;
import net.minecraft.src.ModLoader;

/**
 * Abstract Class for Agriculture Crops
 * Any class extending this abstract can be registered as unique crop via the register-method, to be added into the IC2AC-mechanisms
 * @author Alblaka
 *
 */
public abstract class CropCard
{
	/**
	 * Obviously, name your Plant here. Note: Will be used ingame and displayed to players, don't pick abstract names.
	 */
	public abstract String name();
	
	/**
	 * Override to add your own name here, will be displayed upon analysing seeds.
	 * @return Default is me, since most Crops will be default IC2 stuff :3
	 */
	public String discoveredBy() {return "Alblaka";}
	
	/**
	 * Description of your plant, displayed in Cropnalyser
	 * Keep it short, 2 lines with a few letters each
	 * Default is display of attributes, 2 per line
	 * @param i Line to be printed (0 = first, 1 = second)
	 */
	public String desc(int i)
	{
		String[] att = attributes();
		if (att == null || att.length==0) return "";
		if (i == 0)
		{
			String s = att[0];
			if (att.length >= 2)
			{
				s+=", "+att[1];
				if (att.length >= 3) s+=",";
			}
			return s;
		}
		else
		{
			if (att.length < 3) return "";
			String s = att[2];
			if (att.length >= 4) s+=", "+att[3];
			return s;
		}
	}
	
	/**
	 * Tier represents the complexitiy of a crop. Valuable, powerful crops have high tiers, useless or weak ones have low tiers.
	 * Tier should range from 1 to 16, 0 is Weed
	 */
	public abstract int tier();
	
	/**
	 * Characterize the plants breeding attributes by assigning five values ranging from 0 to 5 for following Stats:
	 * 0: Chemistry (Industrial uses based on chemical plant-components)
	 * 1: Consumeable (Food, Potion ingredients, stuff meant to be eaten (or similaryly used))
	 * 2: Defensive (Plants with defense capabilities (damaging, explosive, chemical) or "abilitys" in general)
	 * 3: Colorful (How colorful/aestically/beautiful is the plant, f.e. dye-plants or plants without actual effects)
	 * 4: Weedish (Is this plant weed-like and rather unwanted/quick-spreading? Rare super-breed plants should have low values here)
	 * @param n The index of the requested stat
	 * @return The requested value of the stats, based on n
	 */
	public abstract int stat(int n);
	/**
	 * Similar to above, here you can specify additional "attributes" of your plant in form of Strings.
	 * Plants sharing same attributes will tend to cross-breed more often.
	 * @return An array of Strings (=attributes)
	 */
	public abstract String[] attributes();
	
	/**
	 * Requests the sprite-index this block is meant to be rendered with.
	 * @param crop Reference to TE-Crop
	 * @return 0-255, representing the sprite index on the casual spritesheet for crops (this cannot be changed)
	 */
	public abstract int getSpriteIndex(TECrop crop);	
	
	/**
	 * Amount of growth points ned to receive +1size
	 * Standard is 200 * tier
	 */
	public int growthDuration(TECrop crop)
	{
		return tier()*200;
	}
	
	/**
	 * Check whether the given crop can grow further.
	 * Consider following:
	 * -Humidity, Nutrient and AirQuality levels
	 * -Already reached size
	 * -Light
	 * -Special biomes / conditions (access via crop.worldObj)
	 * NOTE: This method will be called upon empty upgraded Crops to check whether a neighbouring plant can cross onto it! Thus don't rely on size>0 and id = real
	 * @param crop Reference to TE-Crop
	 * @return true if crop can currently grow
	 */
	public abstract boolean canGrow(TECrop crop);
	
	/**
	 * Use/Override this functions to make plants more depending on one of the three qualities.
	 * Normal behavior is rating the three stats "normal", with each of them ranging from 0-30.
	 * Basic rule: Assume everything returns 10. all together must equal 30. Add Factors to your like. (f.e. *0.7 *0.9 *1.4)
	 * @param humidity Ground humidity, influenced by hydration
	 * @param nutrients Nutrient quality in ground, based on fertilizers
	 * @param air Air quality, influences by open gardens and less crops surrounding this one
	 * @return An int between 0 and 30
	 */
	public int weightInfluences(TECrop crop, float humidity, float nutrients, float air)
	{
		return (int) (humidity+nutrients+air);
	}
	
	/**
	 * Used to determine whether this plant can cross towards another cropblock
	 * Default value is size==3, reduce this number if your plant doesn't grow to that size
	 */
	public boolean canCross(TECrop crop)
	{
		return crop.size >= 3;
	}
	
	
	/**
	 * Called if a crop is rightclicked by a player.
	 * Per default: Will harvest the plant
	 * Override if you need a special behavior
	 * @return true if something changed
	 */
	public boolean rightclick(TECrop crop, EntityPlayer player)
	{
		return crop.harvest(true);
	}
	
	/**
	 * Check whether the crop can be harvested (=fully grown, usually)
	 * @return true if the crop can be harvested in its current state
	 */
	public abstract boolean canBeHarvested(TECrop crop);
	
	/**
	 * Base chance for dropping gains, specify values > 1 to create multiple drops
	 * Standard value is 0.95^tier
	 */
	public float dropGainChance()
	{
		float base = 1F;
		for (int i = 0; i < tier(); i++) {base*=0.95;}
		return base;
	}
	
	/**
	 * Define the drops upon HARVESTING the plant here
	 * Multiple drops of the returned ItemStack CAN occur
	 */
	public abstract ItemStack getGain(TECrop crop);
	
	/**
	 * Define the size of the plant after it got harvested
	 * @return default 1
	 */
	public byte getSizeAfterHarvest(TECrop crop) {return 1;}
	
	
	/**
	 * Called if a crop is leftclicked by a player.
	 * Per default: Will pick the plant, leaving an empty crop.
	 * Override if you need a special behavior
	 * @return true if something changed
	 */
	public boolean leftclick(TECrop crop, EntityPlayer player)
	{
		return crop.pick(true);
	}
		
	/**
	 * Base chance for dropping seeds upon getting picked, additionally to the quite-always dropping seed (plant itself)
	 * Standard value is 0.5*0.8^tier, big minus for size <= 2 crops (prevents speed-crossing)
	 */
	public float dropSeedChance(TECrop crop)
	{
		if (crop.size == 1) return 0;
		float base = 0.5F;
		if (crop.size == 2) base/=2F;
		for (int i = 0; i < tier(); i++) {base*=0.8;}
		return base;
	}
	
	/**
	 * Called if seeds are meant to be dropped
	 * Multiple drops of the returned ItemStack CAN occur
	 * Default: Generates an ItemStack containing one piece of seed from this crop.
	 * Override for special seeddrop behavior
	 */
	public ItemStack getSeeds(TECrop crop)
	{
		return crop.generateSeeds(crop.id, crop.statGrowth, crop.statGain, crop.statResistance, crop.scanLevel);
	}
	
	
	/**
	 * Override this, if you need a notification if a nearby block changes. F.e. for inflameable plants or similar stuff.
	 */
	public void onNeighbourChange(TECrop crop){}
	
	/**
	 * Override this for redstone-emitting plants
	 * @return true if plant is meant to power
	 */
	public boolean emitRedstone(TECrop crop){return false;}
	
	/**
	 * Override this, if you need a notification of the block being destroyed (explosion, otherwise harvest/pick will be used)
	 */
	public void onBlockDestroyed(TECrop crop){}
	
	/**
	 * Override this if you want your plant to emit light.
	 */
	public int getEmittedLight(TECrop crop) {return 0;}
	
	/**
	 * Override this, if you need special effects whenever someone runs into your crop.
	 * Default: True if it's an EntityLiving
	 * @param entity The Entity colliding
	 * @return true if the normal "is crop damaged by somebody trampling it"-calculation should apply, false if it should be skipped (f.e. because your crop just exploded)
	 */
	public boolean onEntityCollision(TECrop crop, Entity entity)
	{
		if (entity instanceof EntityPlayer)
		{
			return !((EntityPlayer)entity).isSneaking();
		}
		return entity instanceof EntityLiving;
	}
	
	
	/**
	 * Called every tick (! depending on tickrate, this usually means every 256 ingame ticks =~12 seconds), override if necessary
	 */
	public void tick(TECrop crop) {}
	
	/**
	 * Check whether this plant spreads Weed (ID0) to surrounding tiles
	 * Default applys for Weed and all crops with a high Growth stat if they reached a size of at least 2
	 */
	public boolean isWeed(TECrop crop)
	{
		return crop.size>=2 && (crop.id==0 || crop.statGrowth>=24);
	}
	
	
	/**
	 * @return ID of this CropCard or -1 if it's not registered
	 */
	public final int getId()
	{
		for (int i = 0; i < cropCardList.length; i++)
		{
			if (this == cropCardList[i])
			{
				return i;
			}
		}
		return -1;
	}
	
	
	/**
	 * Private storage location for all registered CropCards
	 * Access via the "registerCrop" or "getCrop" methods
	 */
	private static final CropCard[] cropCardList = new CropCard[256];
	
	public static int cropCardListLength() {return cropCardList.length;}
	
	/**
	 * Returns the CropCard stored for the given ID.
	 * If the id is invalid (outOfBounds), the first crop (usually weed) will be returned.
	 * If the id is unregistered, same reaction, but there will be a console print to support debugging. If this pops up, double-check your method for registering your CropCards.
	 * @param id Input your cropID here
	 * @return CropCard-Subclass representative for the given id
	 */
	public static final CropCard getCrop(int id)
	{
		if (id < 0 || id >= cropCardList.length)
		{// Out of bounds
			return cropCardList[0];
		}
		if (cropCardList[id]==null)
		{// Out of bounds
			System.out.println("[IndustrialCraft] Something tried to access non-existant cropID #"+id+"!!!");
			return cropCardList[0];
		}
		
		return cropCardList[id];
	}
	
	/**
	 * Can be called to ensure a requested ID exists without triggering the safety print.
	 * @param i ID to be checked
	 * @return true if the the given id is inbounds and the registered slot is not null
	 */
	public static final boolean idExists(int id)
	{
		return !(id < 0 || id >= cropCardList.length || cropCardList[id]==null);
	}
	
	/**
	 * Will search for the first free ID and add the given Crop on that ID
	 * WARNING! Use of this method is DISRECOMMENDED. As IC-intern crops will be loaded first,
	 * any cropCards added by IC itself will probably take over IDs of crops assigned in earlier versions by addons via this method.
	 * Use the second registerCrop instead and only resort to this as "emergency" mechanism.
	 * @param crop CropCard to be registered
	 * @return The id of the crop, -1 if no free slot was found
	 */
	public static final short registerCrop(CropCard crop)
	{
		for (short x = 0; x < cropCardList.length; x++)
		{// Iterate through list
			if (cropCardList[x]==null)
			{// Found empty slot, add crop here
				cropCardList[x]=crop;
				nameReference.addLocal("item.cropSeed"+x+".name", crop.name()+" Seeds");
				return x;
			}
		}
		//No free slot avaible
		return -1;
	}
	
	/**
	 * Will attempt to register the given CropCard with the given id.
	 * If the provided id is invalid (oob) or the id is already taken, the operation will not be performed.
	 * @param crop CropCard to be registered
	 * @param i ID the CropCard is meant to be registered with
	 * @return true if CropCard was successfully registered
	 */
	public static final boolean registerCrop(CropCard crop, int i)
	{
		if (i < 0 || i >= cropCardList.length)
		{// Out of bounds
			return false;
		}
		if (cropCardList[i]==null)
		{
			cropCardList[i]=crop;
			nameReference.addLocal("item.cropSeed"+i+".name", crop.name()+" Seeds");
			return true;
		}
		System.out.println("[IndustrialCraft] Cannot add crop:"+crop.name()+" on ID #"+i+", slot already occupied by crop:"+cropCardList[i].name());
		return false;
	}

	/**
	 * Dont use for anything
	 */
	public static TECrop nameReference;
}
