package net.minecraft.src.ic2.api;

/**
 * @file
 * @author Player
 * @version 1.1
 *
 * @section DESCRIPTION
 *
 * The ElectricItem api class allows to charge, discharge, use etc. IElectricItem items.
 *
 * It's possible to determine the charge/remaining capacity by calling charge/discharge with
 * ignoreTransferlimit = true and simulate = true
 */

import net.minecraft.src.EntityPlayer;
import net.minecraft.src.ItemStack;

public final class ElectricItem {
	/**
	 * Charge an item with a specified amount of energy
	 *
	 * @param itemStack ItemStack containing the IElectricItem
	 * @param amount amount of energy to charge [EU]
	 * @param tier tier of the charging device, has to be at least as high as the item to charge
	 * @param ignoreTransferLimit ignore the transfer limit specified by getTransferLimit()
	 * @param simulate don't actually change the item, just determine the return value
	 * @return energy transfered into the IElectricItem
	 */
	public static int charge(ItemStack itemStack, int amount, int tier, boolean ignoreTransferLimit, boolean simulate) {
		try {
			String packageName = Ic2Recipes.class.getPackage().getName();
			return (Integer) Class.forName(packageName.substring(0, packageName.lastIndexOf('.')) + ".common.ElectricItem").getMethod("charge", ItemStack.class, Integer.TYPE, Integer.TYPE, Boolean.TYPE, Boolean.TYPE).invoke(null, itemStack, amount, tier, ignoreTransferLimit, simulate);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Discharge an item by a specified amount of energy
	 *
	 * @param itemStack ItemStack containing the IElectricItem
	 * @param amount amount of energy to discharge [EU]
	 * @param tier tier of the discharging device, has to be at least as high as the item to discharge
	 * @param ignoreTransferLimit ignore the transfer limit specified by getTransferLimit()
	 * @param simulate don't actually dischange the item, just determine the return value
	 * @return energy retrieved from the IElectricItem
	 */
	public static int discharge(ItemStack itemStack, int amount, int tier, boolean ignoreTransferLimit, boolean simulate) {
		try {
			String packageName = Ic2Recipes.class.getPackage().getName();
			return (Integer) Class.forName(packageName.substring(0, packageName.lastIndexOf('.')) + ".common.ElectricItem").getMethod("discharge", ItemStack.class, Integer.TYPE, Integer.TYPE, Boolean.TYPE, Boolean.TYPE).invoke(null, itemStack, amount, tier, ignoreTransferLimit, simulate);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Helper function to determing if the specified ItemStack contains at least amount EU.
	 *
	 * @param itemStack ItemStack containing the IElectricItem to check
	 * @param amount amount of energy to check [EU]
	 * @return true if there's enough energy
	 *
	 * @note this doesn't take armor (BatPack etc.) into account
	 * @note this is supposed to be used in the item code during operation, e.g. if you want to
	 *       implement your own electric item.
	 */
	public static boolean canUse(ItemStack itemStack, int amount) {
		try {
			String packageName = Ic2Recipes.class.getPackage().getName();
			return (Boolean) Class.forName(packageName.substring(0, packageName.lastIndexOf('.')) + ".common.ElectricItem").getMethod("canUse", ItemStack.class, Integer.TYPE).invoke(null, itemStack, amount);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Helper function to try to retrieve a specified amount of energy from an Item and armor
	 * (BatPack etc.) if available.
	 *
	 * @param itemStack ItemStack containing the IElectricItem
	 * @param amount amount of energy to discharge [EU]
	 * @param player the player holding the Item
	 * @return true if the operation succeeded
	 *
	 * @note this is supposed to be used in the item code during operation, e.g. if you want to
	 *       implement your own electric item.
	 */
	public static boolean use(ItemStack itemStack, int amount, EntityPlayer player) {
		try {
			String packageName = Ic2Recipes.class.getPackage().getName();
			return (Boolean) Class.forName(packageName.substring(0, packageName.lastIndexOf('.')) + ".common.ElectricItem").getMethod("use", ItemStack.class, Integer.TYPE, EntityPlayer.class).invoke(null, itemStack, amount, player);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * Helper function to charge an Item from the armor a player is wearing (BatPack etc.)
	 *
	 * @param itemStack ItemStack containing the IElectricItem
	 * @param player the player holding the Item
	 * @noreturn
	 *
	 * @note this is supposed to be used in the item code during operation, e.g. if you want to
	 *       implement your own electric item.
	 * @note use already contains this functionality.
	 */
	public static void chargeFromArmor(ItemStack itemStack, EntityPlayer player) {
		try {
			String packageName = Ic2Recipes.class.getPackage().getName();
			Class.forName(packageName.substring(0, packageName.lastIndexOf('.')) + ".common.ElectricItem").getMethod("chargeFromArmor", ItemStack.class, EntityPlayer.class).invoke(null, itemStack, player);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
}

