package net.minecraft.src.ic2.api;

import net.minecraft.src.Block;
import net.minecraft.src.ItemStack;
import net.minecraft.src.TileEntity;

/**
 * Interfacing-class between API and TileEntityCrop
 * This class is extended by TileEntityCrop, which contains the codes to handle the whole block-related stuff.
 * It contains all methods you have access to via a CropCard
 * @author Alblaka
 */
public abstract class TECrop extends TileEntity
{
	/**
	 * Stores the id of the crop situated here. Initially crops have -1 = No crop
	 */
	public short id = -1;
	
	/**
	 * Current size of the plant
	 * 1 = Starting size
	 * Maximum size depends on plant
	 */
	public byte size = 0;
	
	/**
	 * Growth ability of a plant
	 * Higher = Faster
	 */
	public byte statGrowth = 0;
	/**
	 * Chances and amount of gain from harvested crops, as well applies to dropped seeds
	 * Higher = More drops
	 */
	public byte statGain = 0;
	/**
	 * Resistance represents crops ability to resist against being trampeled, eaten or similaryly raped
	 * Higher = More resistant
	 */
	public byte statResistance = 0;
	
	/**
	 * If scanned crops are planted, memorize their scan level to display later on
	 */
	public byte scanLevel = 0;
	
	/**
	 * Use this variable to store any additional information into a specific TECrop
	 */
	public short[] custumData = new short[16];
	
	/**
	 * Storage of Nutrients provided
	 * 0 = Nothing, 100 = Full
	 */
	public byte nutrientStorage = 0;
	/**
	 * Storage of Water provided
	 * 0 = Nothing, 1-10 = Basic hydration (normal water), 11-100 = Full Hydration with Destilled Water
	 */
	public byte waterStorage = 0;
	
	/**
	 * Provides current level of humidity for this crop.
	 * Ranges from 0 to 10 (10 = humid)
	 * 
	 * Will be updated every couple of seconds OR if being requested before the first update.
	 */
	public abstract byte getHumidity();
	
	/**
	 * Provides current level of nutrients for the crops
	 * Ranges from 0 to 10 (10 = full of food for the food of our food)
	 * 
	 * Will be updated every couple of seconds OR if being requested before the first update.
	 */
	public abstract byte getNutrients();
	
	/**
	 * Provides current level of fresh air for the crops
	 * Ranges from 0 to 10 (10 = fresh air)
	 * 
	 * Will be updated every couple of seconds OR if being requested before the first update.
	 */
	public abstract byte getAirQuality();
	
	/**
	 * Provides light level of crop
	 */
	public int getLightLevel()
	{
		return worldObj.getBlockLightValue(xCoord, yCoord, zCoord);
	}
	
	
	/**
	 * Pick the given crop, turning it into seeds and remove it from cropblock
	 * @param manual Done by hand (not-automatized)
	 * @return true if successfully picked
	 */
	public abstract boolean pick(boolean manual);
	
	/**
	 * Harvest the given crop, turning it into gain and reducing it's size
	 * @param manual Done by hand (not-automatized)
	 * @return true if successfully harvested
	 */
	public abstract boolean harvest(boolean manual);
	
	/**
	 * Resets the state of the crop growing here to empty
	 * USe this if you want to "clear" the crop
	 */
	public abstract void reset();

	/**
	 * Call this to schedule an update regarding textures and lightning.
	 */
	public abstract void updateState();

	/**
	 * Searches through the blocks below the crop, trying to find the specified block
	 * - Skips the farmland block below crop and iterates through the next 3 blocks
	 * - Stops iteration upon hitting air = false
	 * - Stops iteration upon hitting given block = true
	 * - Otherwise false
	 */
	public abstract boolean isBlockBelow(Block oregold);

	/**
	 * Generates a CropSeed-item with the given values
	 * @param plant Crop-ID
	 * @param growth statGrowth
	 * @param gain statGain
	 * @param resis statResistance
	 * @param scan scanned Level of the crop
	 */
	public abstract ItemStack generateSeeds(short plant, byte growth, byte gain, byte resis, byte scan);
	
	/**
	 * Do not bother with this method. It's a hook for the API to automative localization of plantnames without direct access to IC files.
	 */
	public abstract void addLocal(String s1, String s2);

}
